@@ -137,7 +137,7 @@ def simplify(
 
             new_node = simplify_literals(new_node, root)
             new_node = simplify_equality(new_node)
-            new_node = simplify_parens(new_node)
+            new_node = simplify_parens(new_node, dialect)
             new_node = simplify_datetrunc(new_node, dialect)
             new_node = sort_comparison(new_node)
             new_node = simplify_startswith(new_node)
@@ -729,7 +729,7 @@ def _simplify_binary(expression, a, b):
     return None
 
 
-def simplify_parens(expression):
+def simplify_parens(expression: exp.Expression, dialect: DialectType = None) -> exp.Expression:
     if not isinstance(expression, exp.Paren):
         return expression
 
@@ -737,23 +737,35 @@ def simplify_parens(expression):
     parent = expression.parent
     parent_is_predicate = isinstance(parent, exp.Predicate)
 
+    if isinstance(this, exp.Select):
+        return expression
+
+    if isinstance(parent, (exp.SubqueryPredicate, exp.Bracket)):
+        return expression
+
+    # Handle risingwave struct columns
+    # see https://docs.risingwave.com/sql/data-types/struct#retrieve-data-in-a-struct
     if (
-        not isinstance(this, exp.Select)
-        and not isinstance(parent, (exp.SubqueryPredicate, exp.Bracket))
-        and (
-            not isinstance(parent, (exp.Condition, exp.Binary))
-            or isinstance(parent, exp.Paren)
-            or (
-                not isinstance(this, exp.Binary)
-                and not (isinstance(this, (exp.Not, exp.Is)) and parent_is_predicate)
-            )
-            or (isinstance(this, exp.Predicate) and not parent_is_predicate)
-            or (isinstance(this, exp.Add) and isinstance(parent, exp.Add))
-            or (isinstance(this, exp.Mul) and isinstance(parent, exp.Mul))
-            or (isinstance(this, exp.Mul) and isinstance(parent, (exp.Add, exp.Sub)))
+        dialect == "risingwave"
+        and isinstance(parent, exp.Dot)
+        and (isinstance(parent.right, (exp.Identifier, exp.Star)))
+    ):
+        return expression
+
+    if (
+        not isinstance(parent, (exp.Condition, exp.Binary))
+        or isinstance(parent, exp.Paren)
+        or (
+            not isinstance(this, exp.Binary)
+            and not (isinstance(this, (exp.Not, exp.Is)) and parent_is_predicate)
         )
+        or (isinstance(this, exp.Predicate) and not parent_is_predicate)
+        or (isinstance(this, exp.Add) and isinstance(parent, exp.Add))
+        or (isinstance(this, exp.Mul) and isinstance(parent, exp.Mul))
+        or (isinstance(this, exp.Mul) and isinstance(parent, (exp.Add, exp.Sub)))
     ):
         return this
+
     return expression
 
 
@@ -1576,7 +1588,7 @@ class Gen:
         kvs = []
         arg_types = list(node.arg_types)[arg_index:] if arg_index else node.arg_types
 
-        for k in arg_types or arg_types:
+        for k in arg_types:
             v = node.args.get(k)
 
             if v is not None:
