@@ -1547,6 +1547,18 @@ DOUBLE;
 ASIN(tbl.double_col);
 DOUBLE;
 
+# dialect: snowflake
+ASINH(tbl.double_col);
+DOUBLE;
+
+# dialect: snowflake
+ATAN(tbl.double_col);
+DOUBLE;
+
+# dialect: snowflake
+ATAN2(tbl.double_col, tbl.double_col);
+DOUBLE;
+
 # dialect: snowflake
 ATANH(tbl.double_col);
 DOUBLE;
@@ -1631,6 +1643,22 @@ INT;
 CHARINDEX('world', 'hello world', 1);
 INT;
 
+# dialect: snowflake
+CEIL(3.14);
+DOUBLE;
+
+# dialect: snowflake
+CEIL(3.14::FLOAT, 1);
+FLOAT;
+
+# dialect: snowflake
+CEIL(3.14, 1);
+DOUBLE;
+
+# dialect: snowflake
+CEIL(10::NUMERIC);
+NUMBER;
+
 # dialect: snowflake
 CHAR(65);
 VARCHAR;
@@ -1919,6 +1947,14 @@ INT;
 POSITION('abc', 'abcdef', 1);
 INT;
 
+# dialect: snowflake
+PI();
+DOUBLE;
+
+# dialect: snowflake
+POW(tbl.double_col, 2);
+DOUBLE;
+
 # dialect: snowflake
 LOWER(tbl.str_col);
 VARCHAR;
@@ -2131,6 +2167,14 @@ BIGINT;
 ROUND(tbl.double_col, 0, 'HALF_TO_EVEN');
 DOUBLE;
 
+# dialect: snowflake
+SQUARE(tbl.double_col);
+DOUBLE;
+
+# dialect: snowflake
+TANH(tbl.double_col);
+DOUBLE;
+
 # dialect: snowflake
 RIGHT('hello world', 5);
 VARCHAR;
@@ -2251,6 +2295,14 @@ VARCHAR;
 SIN(tbl.double_col);
 DOUBLE;
 
+# dialect: snowflake
+SINH(1);
+DOUBLE;
+
+# dialect: snowflake
+SINH(1.5);
+DOUBLE;
+
 # dialect: snowflake
 SIGN(tbl.double_col);
 INT;
